
from RciPacket import RciPacket
from RciLog import RciLog

# Request packet are sent from client to server telling the server to change its status or to reply with its configuration data
class RciRequestPacket(RciPacket):
    
    # Base request field parameters
    RCI_REQ_BASE_ID = {
                       'READ_REQUEST'       : 0x00,
                       'WRITE_REQUEST'      : 0x01,
                       'CMD_EXE_REQUEST'    : 0x02
                       }
    
    # Sub request field parameters
    RCI_REQ_SUB_ID_READ = {
                           'RdTxCfg'            : 0x0000,
                           'RdContentMgrCfg'    : 0x0001,
                           'RdTimeCfg'          : 0x0002,
                           'RdTxtMsgPos'        : 0x0010,
                           'RdIsOnAir'          : 0x0020
                           }
    RCI_REQ_SUB_ID_WRITE = {
                           'WrTxCfg'            : 0x0000,
                           'WrContentMgrCfg'    : 0x0001,
                           'WrTimeCfg'          : 0x0002
                           }
    RCI_REQ_SUB_ID_CMD_EXE = {
                           'CmdUpdateTxtMsg'    : 0x0000,
                           'CmdUpdateAwgnSnr'   : 0x0001,
                           'CmdStartTx'         : 0x0002,
                           'CmdStopTx'          : 0x0003,
                           'CmdReconfigureTx'   : 0x0004
                           }

    log = RciLog("RciRequestPacket")
    
    def __init__(self):
        RciPacket.__init__(self,'RCI_REQUEST_PID')

    # Define a class to manage different packet request type
    class RequestType():
        
        RCI_REQ_NBITS_BASE = 8
        RCI_REQ_NBITS_SUB = 16
        
        def __init__(self,bid, sid):
            if bid >= ( 1 << self.RCI_REQ_NBITS_BASE ):
                raise ValueError ("Request type base ID out of bounds")
            
            if sid >= ( 1 << self.RCI_REQ_NBITS_SUB ):
                raise ValueError ("Request type sub ID out of bounds")
                
            self.base_id = bid
            self.sub_id = sid
        
        def getBaseId(self):
            return self.base_id
        
        def getSubId(self):
            return self.sub_id
        
        def getRequestPacketDescriptor(self):
            return ((self.base_id & 0xFF) << self.RCI_REQ_NBITS_SUB) | (self.sub_id & 0xFFFF)
    
    # Dictionary of different RciRequestPacket Type
    RciRequestPacketType = {
                            'RdTxCfg'           : RequestType(RCI_REQ_BASE_ID['READ_REQUEST'],RCI_REQ_SUB_ID_READ['RdTxCfg']),
                            'RdContentMgrCfg'   : RequestType(RCI_REQ_BASE_ID['READ_REQUEST'],RCI_REQ_SUB_ID_READ['RdContentMgrCfg']),
                            'RdTimeCfg'         : RequestType(RCI_REQ_BASE_ID['READ_REQUEST'],RCI_REQ_SUB_ID_READ['RdTimeCfg']),
                            'RdTxtMsgPos'       : RequestType(RCI_REQ_BASE_ID['READ_REQUEST'],RCI_REQ_SUB_ID_READ['RdTxtMsgPos']),
                            'RdIsOnAir'         : RequestType(RCI_REQ_BASE_ID['READ_REQUEST'],RCI_REQ_SUB_ID_READ['RdIsOnAir']),
                            'WrTxCfg'           : RequestType(RCI_REQ_BASE_ID['WRITE_REQUEST'],RCI_REQ_SUB_ID_WRITE['WrTxCfg']),
                            'WrContentMgrCfg'   : RequestType(RCI_REQ_BASE_ID['WRITE_REQUEST'],RCI_REQ_SUB_ID_WRITE['WrContentMgrCfg']),
                            'WrTimeCfg'         : RequestType(RCI_REQ_BASE_ID['WRITE_REQUEST'],RCI_REQ_SUB_ID_WRITE['WrTimeCfg']),
                            'CmdUpdateTxtMsg'   : RequestType(RCI_REQ_BASE_ID['CMD_EXE_REQUEST'],RCI_REQ_SUB_ID_CMD_EXE['CmdUpdateTxtMsg']),
                            'CmdUpdateAwgnSnr'  : RequestType(RCI_REQ_BASE_ID['CMD_EXE_REQUEST'],RCI_REQ_SUB_ID_CMD_EXE['CmdUpdateAwgnSnr']),
                            'CmdStartTx'        : RequestType(RCI_REQ_BASE_ID['CMD_EXE_REQUEST'],RCI_REQ_SUB_ID_CMD_EXE['CmdStartTx']),
                            'CmdStopTx'         : RequestType(RCI_REQ_BASE_ID['CMD_EXE_REQUEST'],RCI_REQ_SUB_ID_CMD_EXE['CmdStopTx']),
                            'CmdReconfigureTx'  : RequestType(RCI_REQ_BASE_ID['CMD_EXE_REQUEST'],RCI_REQ_SUB_ID_CMD_EXE['CmdReconfigureTx'])
                          }
    
    def getRequestType(self):
        for req in self.RciRequestPacketType.values() :
            if req.getRequestPacketDescriptor() == self.getPacketDescriptor():
                return req
        return None
    
    def setRequestPacketDescriptor(self,reqType):
        self.setPacketDescriptor(reqType.getRequestPacketDescriptor())
    
    def getBaseTypeID(self):
        return (self.getPacketDescriptor() >> self.RequestType.RCI_REQ_NBITS_SUB) & 0xFF

    def getSubTypeId(self):
        return (self.getPacketDescriptor() & 0xFFFF)
        
        
            
        