from RciClient import RciClient
from RciRequestPacket import RciRequestPacket
from RciReplyPacket import RciReplyPacket
from RciLog import RciLog
from XmlElement import XmlElement
import string 
from array import array
import RciPacket
import codecs
import xml.etree.ElementTree as ET
from struct import unpack_from
from sys import argv
from getopt import getopt


class SparkController():

    def __init__(self):
        self.rciClient = RciClient()
        self.clientRequest = RciRequestPacket()
        self.clientReply = RciReplyPacket()
        self.log = RciLog("SparkController")
        self.false= 0x00
        self.true = 0x01
    # Open the RCI connection 
    # host : hostname or IP address
    # port : port number
    def connect(self, host, port):
        self.rciClient.connect(host, port)

    # Close the RCI connection
    def disConnect(self):
        self.rciClient.disConnect()

    # Send a command to the spark server to initiate
    # a broadcast with the current configuration
    # NB : Command request does never carry any payload
    def cmdStartTx(self):
        try:
            self.log.write("**********Command cmdStartTx**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['CmdStartTx']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
        except Exception:
            raise Exception


    # Send a command to the spark server to stop
    # the current broadcast
    # NB : Command request does never carry any payload
    def cmdStopTx(self):
        try:
            self.log.write("**********Command cmdStopTx**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['CmdStopTx']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
        except Exception:
            raise Exception

    # Send a command to the spark server to reconfigure 
    # the current broadcast with the new configuration
    # NB : Command request does never carry any payload
    def cmdReconfigureTx(self):
        try:
            self.log.write("**********Command cmdReconfigureTx**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['CmdReconfigureTx']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
        except Exception:
            raise Exception

    # Send a command to the spark server to replace the
    # text message queue of the active broadcast with the current
    # configuration
    # NB : Command request does never carry any payload
    def cmdUpdateTxtMsg(self):
        try:
            self.log.write("**********Command cmdUpdateTxtMsg**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['CmdUpdateTxtMsg']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
        except Exception:
            raise Exception

    # Send a command to the spark server to update the 
    # SNR value for AWGN generation with the current configuration 
    # NB : Command request does never carry any payload
    def cmdUpdateAwgnSnr(self):
        try:
            self.log.write("**********Command cmdUpdateAwgnSnr**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['CmdUpdateAwgnSnr']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
        except Exception:
            raise Exception

    # Send a read command to the server and waits
    # for the server to reply with the transmitter configuration
    # return a root XML element tree of transmitter configuration
    def rdTxCfg(self):
        try:
            self.log.write("**********Command rdTxCfg**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['RdTxCfg']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
            self.log.write("rdTxCfg command return file : " + self.readXMLStringFromPayload(self.clientReply)+"\n", 'DEBUG')
            return self.convertStringToXml(self.readXMLStringFromPayload(self.clientReply))
        except Exception:
            raise Exception

    # Send a read command to the server and waits
    # for the server to reply with the content manager configuration
    # return a root XML element tree of content manager configuration
    def rdContentMgr(self):
        try:
            self.log.write("**********Command rdContentMgr**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['RdContentMgrCfg']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
            self.log.write("rdContentMgr command return file : " + self.readXMLStringFromPayload(self.clientReply)+"\n", 'DEBUG')
            return self.convertStringToXml(self.readXMLStringFromPayload(self.clientReply))
        except Exception:
            raise Exception

    # Send a read command to the server and waits
    # for the server to reply with the NTP time configuration
    # return a root XML element tree of NTP time configuration
    def rdTimeCfg(self):
        try:
            self.log.write("**********Command rdTimeCfg**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
            requestType = self.clientRequest.RciRequestPacketType['RdTimeCfg']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
            self.log.write("rdTimeCfg command return file : " + self.readXMLStringFromPayload(self.clientReply)+"\n", 'DEBUG')
            return self.convertStringToXml(self.readXMLStringFromPayload(self.clientReply))
        except Exception:
            raise Exception

    # Send a read command to the server and waits
    # for the server to reply with the position of the text
    # message being transmitted. The value refers to the index in 
    # the text message queue of the content manager.
    # return uint32
    def rdTxtMsgPos(self):
        try:
            self.log.write("**********Command rdTxtMsgPos**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()
    
            requestType = self.clientRequest.RciRequestPacketType['RdTxtMsgPos']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
            if self.clientReply.getNumBytesPayload() == 0:
                self.log.write("NO msg position on payload", 'WARNING')
                return
            reply=unpack_from('!I',self.clientReply.getPayload(),0)[0]
            if self.clientReply.getNumBytesPayload() < self.clientReply.SIZEOF_UINT:
                self.log.write("Sizeof payload must be " + str(self.clientReply.SIZEOF_UINT )+ " bytes", 'WARNING')
                raise BufferError
            return reply
        except BufferError:
            raise BufferError("rdTxtMsgPos :  BufferError catched")

    # Sends a read command to the server and waits
    # for the spark server to get back with its broadcast status
    # return true, if spark is broadcasting, false otherwise
    def rdIsOnAir(self):
        try:
            self.log.write("**********Command rdIsOnAir**********", 'NOTIFICATION')
            self.clientRequest.clearPacket()
            self.clientReply.clearPacket()

            requestType = self.clientRequest.RciRequestPacketType['RdIsOnAir']
            self.clientRequest.setRequestPacketDescriptor(requestType)
            self.rciClient.transmit(self.clientRequest, self.clientReply)
            if self.clientReply.getNumBytesPayload() == 0:
                self.log.write("NO IsOnAir information on payload", 'WARNING')
                return
            reply=unpack_from('!B',self.clientReply.getPayload(),0)[0]
            if self.clientReply.getNumBytesPayload() != self.clientReply.SIZEOF_UBYTE:
                self.log.write("Sizeof payload must be " + self.clientReply.SIZEOF_UBYTE+ " byte", 'WARNING')
                raise BufferError
            elif reply != self.false and reply != self.true:
                self.log.write("Illegal value for boolean : " + str(reply) , 'WARNING')
                raise ValueError
            return reply

        except ValueError:
            raise ValueError("rdIsOnAir :ValueError catched ")
        except BufferError:
            raise BufferError("rdIsOnAir :BufferError catched ")

    # Send a write command to the server and transmit
    # the transmitter configuration in the payload field
    # rootTxCfgXml : A root XML element tree with the new transmitter configuration
    def wrTxCfg(self, rootTxCfgXml):
        try:
            self.log.write("**********Command wrTxCfg**********", 'NOTIFICATION')
            if rootTxCfgXml == None:
                self.log.write("Can't write empty rootTxCfgXml", 'ERROR')
                raise Exception
            else:
                self.clientRequest.clearPacket()
                self.clientReply.clearPacket()
                trXml=self.convertXmlToString(rootTxCfgXml)
                requestType = self.clientRequest.RciRequestPacketType['WrTxCfg']
                self.clientRequest.setRequestPacketDescriptor(requestType)
                self.writeXMLStringToPayload(self.clientRequest,trXml)
                self.log.write("wrTxCfg command send file : " + trXml +"\n", 'DEBUG')
                self.rciClient.transmit(self.clientRequest,self.clientReply)
        except Exception:
            raise Exception

    # Send a write command to the server and transmit
    # the content manager configuration in the payload field
    # cmXml : A root XML element tree with the new content manager configuration
    def wrContentMgr(self,rootContentMgrXml):
        try:
            self.log.write("**********Command WrContentMgr**********", 'NOTIFICATION')
            if rootContentMgrXml == None:
                self.log.write("Can't write empty rootContentMgrXml", 'ERROR')
                raise Exception
            else:
                self.clientRequest.clearPacket()
                self.clientReply.clearPacket()
                cmXml=self.convertXmlToString(rootContentMgrXml)
                requestType = self.clientRequest.RciRequestPacketType['WrContentMgrCfg']
                self.clientRequest.setRequestPacketDescriptor(requestType)
                self.writeXMLStringToPayload(self.clientRequest,cmXml)
                self.log.write("wrContentMgr command send file : " + cmXml +"\n", 'DEBUG')
                self.rciClient.transmit(self.clientRequest,self.clientReply)
        except Exception:
            print  type(Exception)
            raise Exception

    # Send a write command to the server and transmit
    # the NTP time configuration in the payload field
    # timeXML : A root XML element tree with the new NTP time configuration
    def wrTimeCfg(self,rootTimeXML):
        try:
            self.log.write("**********Command WrTimeCfg**********", 'NOTIFICATION')
            if rootTimeXML == None:
                self.log.write("Can't write empty rootTimeXML", 'ERROR')
                raise Exception
            else:
                self.clientRequest.clearPacket()
                self.clientReply.clearPacket()
                timeXml=self.convertXmlToString(rootTimeXML)
                requestType = self.clientRequest.RciRequestPacketType['WrTimeCfg']
                self.clientRequest.setRequestPacketDescriptor(requestType)
                self.writeXMLStringToPayload(self.clientRequest,timeXml)
                self.log.write("wrTimeCfg command send file : " + timeXml +"\n", 'DEBUG')
                self.rciClient.transmit(self.clientRequest,self.clientReply)
        except Exception:
            raise Exception

    # Converts an XML string into a UTF-8 byte array and fills
    # the data into the payload field of a packet
    # requestPacket: The RciPacket which will carry the payload
    # xmlString : The XML string from which the payload will be generated.
    def writeXMLStringToPayload(self,requestPacket,xmlString):
        try:
            xmlBuffer=array('c')
            xmlBuffer.fromstring(xmlString.strip())
            numBytes=len(xmlBuffer)
            if numBytes > RciPacket.RciPacket.RCI_MAX_NUM_BYTES_PAYLOAD :
                raise StandardError("Too many chars in XML string")
            requestPacket.setPayload(xmlBuffer)
            requestPacket.setNumBytesPayload(numBytes)
        except Exception as e :
            self.log.write("writeXMLStringToPayload : Catch exception:",'ERROR')
            raise e

    # Converts an XML UTF-8 byte array of the received RciPacket 
    # into a pyhton string
    # replyPacket : Receive packet with XML payload
    def  readXMLStringFromPayload(self,replyPacket):
        try:
            if replyPacket.getNumBytesPayload() != 0 :
                xmlString = replyPacket.getPayload().tostring()
            else :
                self.log.write("Empty XML string does not much with Spark RCI protocol", 'ERROR')
                raise Exception
            return xmlString  
        except Exception as e:
            self.log.write("readXMLStringFromPayload : Catch exception:",'ERROR')
            raise e

    # Generate the root XML node from an UTF-8 coded XML input string
    # xmlString : The XML input string containing the XmlRootNode
    # return The Xml Root Node
    def convertStringToXml(self,xmlString):
        try:
            if len(xmlString)== 0:
                self.log.write("Empty XML string can not be decoded", 'ERROR')
#             decodedXmlString=xmlString.strip().decode('utf8')
#             root=ET.fromstring(decodedXmlString)
            root=ET.fromstring(xmlString)
            if root.tag != XmlElement.XML_ROOT_ELEMENT:
                raise RuntimeError("Root Element does not much with Spark RCI protocol")
            return XmlElement(root)
        except Exception as e:
            self.log.write("convertStringToXml : Catch exception:",'ERROR')
            print type(e)
            raise e

    # Generates an XML string from the root XML node element. 
    # return The XML string representation of the node document.
    def convertXmlToString(self,rootElement):
        try:
            string=ET.tostring(rootElement.xmlElement,encoding=XmlElement.XML_CHAR_ENCODING,method="xml")
            if string == '':
                raise RuntimeError("Empty XML string does not much with Spark RCI protocol")
            return string
        except Exception as e:
            self.log.write("convertXmlToString : Catch exception:",'ERROR')
            raise e

def printHelp():
    print "This program takes options and arguments as parameters."
    print "Use : python SparkController.py options args"
    print "Options are :\n"
    print "              --ip_host : IP address of the host"
    print "              --port    : Port of the host"
    print "\nArguments are :\n"
    print "\tcmdStartTx       : Start the broadcast\n"
    print "\tcmdStopTx        : Stop an active broadcast\n"
    print "\tcmdReconfigureTx : Reconfigure an active broadcast with the current configuration\n"
    print "\tcmdUpdateTxtMsg  : Replace the text message queue of the active broadcast with the current configuration\n"
    print "\tcmdUpdateAwgnSnr : Update the SNR value for AWGN generation with the current configuration\n"
    print "\trdTxCfg          : Request the transmitter configuration in XML notation\n"
    print "\trdContentMgr     : Request the content manager configuration in XML notation\n"
    print "\trdTimeCfg        : Request the NTP time server configuration in XML notation\n"
    print "\trdTxtMsgPos      : Request the position of the text message being transmitted."
    print "\t                   The value refers to the index in the text message queue of the content manager\n"
    print "\trdIsOnAir        : Request Spark to send if a broadcast is on air or not\n"
    print "\twrTxCfg          : Send the transmitter configuration in XML notation to Spark\n"
    print "\twrContentMgr     : Send the content manager configuration in XML notation to Spark\n"
    print "\twrTimeCfg        : Send the NTP time server configuration in XML notation to Spark\n"

if __name__ == "__main__":

    if len(argv) == 1:
        printHelp()
    else:
        RCI_HOST = '172.20.200.128'
        RCI_PORT = 8500
        options,args=getopt(argv[1:],"",["ip_host=","port="])
        # Set RCI HOST and RCI PORT
        for opt,value in options:
            if opt == "--ip_host":
                if value != "":
                    RCI_HOST=value
            if opt== "--port":
                if value != "":
                    RCI_PORT=int(value)
                    break
        print "RCI_HOST = " + str(RCI_HOST)
        print "RCI_PORT = " + str(RCI_PORT)

        onAir=False
        try:
            # Instantiate spark controller
            sparkController = SparkController()
            # Initialize RCI
            if sparkController.rciClient.isConnected() == True:
                sparkController.disConnect()
            sparkController.connect(RCI_HOST, RCI_PORT)

            if "rdTxCfg" in argv:
                print "\n****************************** Receive Transmitter configuration from server ******************************\n"
                # Receive configuration from server 
                rootTxCfgXml=sparkController.rdTxCfg()
                # Read transmission configuration       
                try:
                    print "Drm current Robustness mode: " + rootTxCfgXml.getChild("txparams").getChild("drm").getChildTextContent("robmode")
                    print "Drm current OFDM bandwith: " + rootTxCfgXml.getChild("txparams").getChild("drm").getChildTextContent("specocc")
                    drmProtectionList=rootTxCfgXml.getChild("txparams").getChild("drm").getChildren("protlevel")
                    print "Drm current MSC LPP Protection: " + drmProtectionList.getNodeByAttribute("type", "lpp").getTextContent()
                    print "Drm current MSC HPP Protection mode: " + drmProtectionList.getNodeByAttribute("type", "hpp").getTextContent()
     
           
                except Exception as e:
                    print e 
                    raise e
    
            if  "wrTxCfg" in argv and "rdTxCfg" in argv:
                print "\n****************************** Send new Transmitter configuration to server ******************************\n"
                # Modify XML stream tree according to test
                try: 
                    rootTxCfgXml.getChild("txparams").getChild("drm").getChild("robmode").setTextContent("Mode B")
                    rootTxCfgXml.getChild("txparams").getChild("drm").getChild("specocc").setTextContent("4")
                    drmProtectionList.getNodeByAttribute("type", "hpp").setTextContent("1") ;
                    # Send modifications back to server
                    sparkController.wrTxCfg(rootTxCfgXml)
    
                except Exception as e:
                    print e
                    raise e
    
            if "rdTimeCfg" in argv:
                print "\n****************************** Receive Time configuration from server ******************************\n"
                # Receive configuration from server 
                rootTimeCfgXml=sparkController.rdTimeCfg()
    
            if "rdTimeCfg" in argv and "wrTimeCfg" in argv:
                print "\n****************************** Send Time configuration to server ******************************\n"
                # Receive configuration from server 
                sparkController.wrTimeCfg(rootTimeCfgXml)
    
            if "rdContentMgr" in argv:
                print "\n****************************** Receive content manager from server ******************************\n"
                # Receive configuration from server 
                rootContentMgrXml=sparkController.rdContentMgr()
                # Read content manger          
                try:
                    drmStreamList=rootContentMgrXml.getChild("contentmgr").getChild("drm").getChild("mux").getChildren("stream")
                     
                    for stream in range(drmStreamList.NodesNumber):
                        print "High protected part for stream " + str(stream) + " = " + str(drmStreamList.getNodeByIndex(stream).getAttrValue("hppbytes"))
                        print "Low protected part for stream  " + str(stream) + " = " + str(drmStreamList.getNodeByIndex(stream).getAttrValue("lppbytes"))
    
                    print 
         
                    drmServicesList=rootContentMgrXml.getChild("contentmgr").getChild("drm").getChild("services").getChildren("service")
                    for svc in range(drmServicesList.NodesNumber):
                        print "Service name for svc     " + str(svc) + " = " + str(drmServicesList.getNodeByIndex(svc).getChildTextContent("label"))
                        print "Service short id for svc " + str(svc) + " = " + str(drmServicesList.NodeList[svc].getAttrValue("shortid"))
                        print
         
                except Exception as e:
                    print e 
                    raise e
    
            if  "wrContentMgr" in argv and "rdContentMgr" in argv:
                print "\n****************************** Send new content manager to server ******************************\n"
                # Modify XML stream tree according to test
                try: 
                    tmp =drmStreamList.getNodeByAttribute("id", "0") ;
                    if tmp  != None:
                        tmp.setAttrValue("lppbytes", "300")
                        tmp.setAttrValue("hppbytes", "100")
                        # Send modifications back to server
                        sparkController.wrContentMgr(rootContentMgrXml)
    
                        print "\n****************************** Receive content manager from server ******************************\n"
                        # Receive configuration from server 
                        rootContentMgrXml=sparkController.rdContentMgr()
    
                        # Read content manger          
                        try:
                            drmStreamList=rootContentMgrXml.getChild("contentmgr").getChild("drm").getChild("mux").getChildren("stream")
                             
                            for stream in range(drmStreamList.NodesNumber):
                                print "High protected part for stream " + str(stream) + " = " + str(drmStreamList.getNodeByIndex(stream).getAttrValue("hppbytes"))
                                print "Low protected part for stream  " + str(stream) + " = " + str(drmStreamList.getNodeByIndex(stream).getAttrValue("lppbytes"))
                            print 
    
                        except Exception as e:
                            print e 
                            raise e
                except Exception as e:
                    print e
                    raise e
    
            if "rdIsOnAir" in argv:
                print "\n****************************** Read from the server if it is on air or not  ******************************\n"
                onAir=sparkController.rdIsOnAir()
                if onAir:
                    print "========> ON AIR"
                else:
                    print "========> NOT ON AIR"
    
            if "rdTxtMsgPos" in argv:
                print "\n****************************** Request position of text msg being transmitted ******************************\n"
                position=sparkController.rdTxtMsgPos()
                print "========> Current posititon = " + str(position)
    
            if "cmdStartTx" in argv:
                print "\n***************************** Command the server to start transmission ******************************\n"
                sparkController.cmdStartTx()
    
            if "cmdStopTx" in argv:
                print "\n***************************** Command the server to stop transmission ******************************\n"
                sparkController.cmdStopTx()
    
            if "cmdReconfigureTx" in argv:
                print "\n***************************** Command the server to reconfigure an active broadcast ******************************\n"
                sparkController.cmdReconfigureTx()
    
            if "cmdUpdateAwgnSnr" in argv:
                print "\n***************************** Command the server to update the SNR value for AWGN ******************************\n"
                rootTxCfgXml=sparkController.rdTxCfg()
                rootTxCfgXml.getChild("txparams").getChild("ofdm").getChild("impairment").getChild("enabled").setTextContent("true")
                rootTxCfgXml.getChild("txparams").getChild("ofdm").getChild("impairment").getChild("cnr").setTextContent("35.2")
                sparkController.wrTxCfg(rootTxCfgXml)
                sparkController.cmdReconfigureTx()
    
            if "cmdUpdateTxtMsg" in argv:
                print "\n***************************** Command the server to update the text message queue of the active broadcast******************************\n"
                sparkController.cmdUpdateTxtMsg()
    
        except Exception as e:
            print "Main: Exception catched"
    
        except BufferError as b:
            print "Main : BufferError catched"
    
        except ValueError as v:
            print "Main : ValueError catched"
    
        sparkController.disConnect()
